@@ -100,6 +100,8 @@ struct blitter_context
    /* Whether the blitter is running. */
    bool running;
 
+   bool use_index_buffer;
+
    /* Private members, really. */
    struct pipe_context *pipe; /**< pipe context */
 
@@ -116,6 +118,7 @@ struct blitter_context
    bool skip_viewport_restore;
    bool is_sample_mask_saved;
    unsigned saved_sample_mask;
+   unsigned saved_min_samples;
 
    unsigned saved_num_sampler_states;
    void *saved_sampler_states[PIPE_MAX_SAMPLERS];
@@ -135,6 +138,10 @@ struct blitter_context
    struct pipe_query *saved_render_cond_query;
    uint saved_render_cond_mode;
    bool saved_render_cond_cond;
+
+   boolean saved_window_rectangles_include;
+   unsigned saved_num_window_rectangles;
+   struct pipe_scissor_state saved_window_rectangles[PIPE_MAX_WINDOW_RECTANGLES];
 };
 
 /**
@@ -148,6 +155,10 @@ struct blitter_context *util_blitter_create(struct pipe_context *pipe);
 void util_blitter_destroy(struct blitter_context *blitter);
 
 void util_blitter_cache_all_shaders(struct blitter_context *blitter);
+void *util_blitter_get_noop_blend_state(struct blitter_context *blitter);
+void *util_blitter_get_noop_dsa_state(struct blitter_context *blitter);
+void *util_blitter_get_discard_rasterizer_state(struct blitter_context *blitter);
+
 
 /**
  * Return the pipe context associated with a blitter context.
@@ -198,7 +209,8 @@ void util_blitter_clear(struct blitter_context *blitter,
                         unsigned width, unsigned height, unsigned num_layers,
                         unsigned clear_buffers,
                         const union pipe_color_union *color,
-                        double depth, unsigned stencil);
+                        double depth, unsigned stencil,
+                        bool msaa);
 
 /**
  * Check if the blitter (with the help of the driver) can blit between
@@ -259,7 +271,8 @@ void util_blitter_blit_generic(struct blitter_context *blitter,
                                unsigned src_width0, unsigned src_height0,
                                unsigned mask, unsigned filter,
                                const struct pipe_scissor_state *scissor,
-                               bool alpha_blend);
+                               bool alpha_blend, bool sample0_only,
+                               unsigned dst_sample);
 
 void util_blitter_blit(struct blitter_context *blitter,
 		       const struct pipe_blit_info *info);
@@ -288,17 +301,6 @@ void util_blitter_default_src_texture(struct blitter_context *blitter,
                                       struct pipe_resource *src,
                                       unsigned srclevel);
 
-/**
- * Copy data from one buffer to another using the Stream Output functionality.
- * 4-byte alignment is required, otherwise software fallback is used.
- */
-void util_blitter_copy_buffer(struct blitter_context *blitter,
-                              struct pipe_resource *dst,
-                              unsigned dstx,
-                              struct pipe_resource *src,
-                              unsigned srcx,
-                              unsigned size);
-
 /**
  * Clear the contents of a buffer using the Stream Output functionality.
  * 4-byte alignment is required.
@@ -384,6 +386,21 @@ void util_blitter_custom_resolve_color(struct blitter_context *blitter,
                                        void *custom_blend,
                                        enum pipe_format format);
 
+/* Used by vc4 for 8/16-bit linear-to-tiled blits */
+void util_blitter_custom_shader(struct blitter_context *blitter,
+                                struct pipe_surface *dstsurf,
+                                void *custom_vs, void *custom_fs);
+
+/* Used by D3D12 for non-MSAA -> MSAA stencil blits */
+void util_blitter_stencil_fallback(struct blitter_context *blitter,
+                                   struct pipe_resource *dst,
+                                   unsigned dst_level,
+                                   const struct pipe_box *dstbox,
+                                   struct pipe_resource *src,
+                                   unsigned src_level,
+                                   const struct pipe_box *srcbox,
+                                   const struct pipe_scissor_state *scissor);
+
 /* The functions below should be used to save currently bound constant state
  * objects inside a driver. The objects are automatically restored at the end
  * of the util_blitter_{clear, copy_region, fill_region} functions and then
@@ -539,10 +556,11 @@ util_blitter_save_so_targets(struct blitter_context *blitter,
 
 static inline void
 util_blitter_save_sample_mask(struct blitter_context *blitter,
-                              unsigned sample_mask)
+                              unsigned sample_mask, unsigned min_samples)
 {
    blitter->is_sample_mask_saved = true;
    blitter->saved_sample_mask = sample_mask;
+   blitter->saved_min_samples = min_samples;
 }
 
 static inline void
@@ -556,6 +574,21 @@ util_blitter_save_render_condition(struct blitter_context *blitter,
    blitter->saved_render_cond_cond = condition;
 }
 
+static inline void
+util_blitter_save_window_rectangles(struct blitter_context *blitter,
+                                    boolean include,
+                                    unsigned num_rectangles,
+                                    const struct pipe_scissor_state *rects)
+{
+   blitter->saved_window_rectangles_include = include;
+   blitter->saved_num_window_rectangles = num_rectangles;
+   if (num_rectangles > 0) {
+      assert(num_rectangles < ARRAY_SIZE(blitter->saved_window_rectangles));
+      memcpy(blitter->saved_window_rectangles, rects,
+             sizeof(*rects) * num_rectangles);
+   }
+}
+
 void util_blitter_common_clear_setup(struct blitter_context *blitter,
                                      unsigned width, unsigned height,
                                      unsigned clear_buffers,
@@ -571,6 +604,36 @@ void util_blitter_restore_fb_state(struct blitter_context *blitter);
 void util_blitter_restore_textures(struct blitter_context *blitter);
 void util_blitter_restore_constant_buffer_state(struct blitter_context *blitter);
 
+/* These are supported combinations of blits from ZS to color and vice versa.
+ * The blitter will do the packing/unpacking of depth and stencil
+ * in the fragment shader.
+ */
+static inline enum pipe_format
+util_blitter_get_color_format_for_zs(enum pipe_format format)
+{
+   switch (format) {
+   case PIPE_FORMAT_Z16_UNORM:
+      return PIPE_FORMAT_R16_UNORM;
+
+   case PIPE_FORMAT_Z32_FLOAT:
+      return PIPE_FORMAT_R32_FLOAT;
+
+   case PIPE_FORMAT_Z24_UNORM_S8_UINT:
+   case PIPE_FORMAT_Z24X8_UNORM:
+   case PIPE_FORMAT_S8_UINT_Z24_UNORM:
+   case PIPE_FORMAT_X8Z24_UNORM:
+      return PIPE_FORMAT_R32_UINT;
+
+   case PIPE_FORMAT_Z32_FLOAT_S8X24_UINT:
+      return PIPE_FORMAT_R32G32_UINT;
+
+   case PIPE_FORMAT_Z32_UNORM:
+   default:
+      assert(0);
+   }
+   return PIPE_FORMAT_NONE;
+}
+
 #ifdef __cplusplus
 }
 #endif
