@@ -221,18 +221,12 @@ bool expr_handler::fold_setcc(alu_node &n) {
 	} else if (isc1) {
 		if (cmp_type == AF_FLOAT_CMP) {
 			if (n.bc.src[0].abs && !n.bc.src[0].neg) {
-				if (cv1.f < 0.0f && (cc == AF_CC_GT || cc == AF_CC_NE)) {
-					cond_result = true;
-					have_result = true;
-				} else if (cv1.f <= 0.0f && cc == AF_CC_GE) {
+				if (cv1.f < 0.0f && cc == AF_CC_NE) {
 					cond_result = true;
 					have_result = true;
 				}
 			} else if (n.bc.src[0].abs && n.bc.src[0].neg) {
-				if (cv1.f > 0.0f && (cc == AF_CC_GE || cc == AF_CC_E)) {
-					cond_result = false;
-					have_result = true;
-				} else if (cv1.f >= 0.0f && cc == AF_CC_GT) {
+				if (cv1.f > 0.0f && cc == AF_CC_E) {
 					cond_result = false;
 					have_result = true;
 				}
@@ -244,18 +238,12 @@ bool expr_handler::fold_setcc(alu_node &n) {
 	} else if (isc0) {
 		if (cmp_type == AF_FLOAT_CMP) {
 			if (n.bc.src[1].abs && !n.bc.src[1].neg) {
-				if (cv0.f <= 0.0f && cc == AF_CC_GT) {
-					cond_result = false;
-					have_result = true;
-				} else if (cv0.f < 0.0f && (cc == AF_CC_GE || cc == AF_CC_E)) {
+				if (cv0.f < 0.0f && (cc == AF_CC_E)) {
 					cond_result = false;
 					have_result = true;
 				}
 			} else if (n.bc.src[1].abs && n.bc.src[1].neg) {
-				if (cv0.f >= 0.0f && cc == AF_CC_GE) {
-					cond_result = true;
-					have_result = true;
-				} else if (cv0.f > 0.0f && (cc == AF_CC_GT || cc == AF_CC_NE)) {
+				if (cv0.f > 0.0f && cc == AF_CC_NE) {
 					cond_result = true;
 					have_result = true;
 				}
@@ -326,7 +314,7 @@ void expr_handler::apply_alu_src_mod(const bc_alu &bc, unsigned src,
 	const bc_alu_src &s = bc.src[src];
 
 	if (s.abs)
-		v = fabs(v.f);
+		v = fabsf(v.f);
 	if (s.neg)
 		v = -v.f;
 }
@@ -424,21 +412,21 @@ bool expr_handler::fold_alu_op1(alu_node& n) {
 	apply_alu_src_mod(n.bc, 0, cv);
 
 	switch (n.bc.op) {
-	case ALU_OP1_CEIL: dv = ceil(cv.f); break;
+	case ALU_OP1_CEIL: dv = ceilf(cv.f); break;
 	case ALU_OP1_COS: dv = cos(cv.f * 2.0f * M_PI); break;
-	case ALU_OP1_EXP_IEEE: dv = exp2(cv.f); break;
-	case ALU_OP1_FLOOR: dv = floor(cv.f); break;
+	case ALU_OP1_EXP_IEEE: dv = exp2f(cv.f); break;
+	case ALU_OP1_FLOOR: dv = floorf(cv.f); break;
 	case ALU_OP1_FLT_TO_INT: dv = (int)cv.f; break; // FIXME: round modes ????
-	case ALU_OP1_FLT_TO_INT_FLOOR: dv = (int32_t)floor(cv.f); break;
-	case ALU_OP1_FLT_TO_INT_RPI: dv = (int32_t)floor(cv.f + 0.5f); break;
-	case ALU_OP1_FLT_TO_INT_TRUNC: dv = (int32_t)trunc(cv.f); break;
+	case ALU_OP1_FLT_TO_INT_FLOOR: dv = (int32_t)floorf(cv.f); break;
+	case ALU_OP1_FLT_TO_INT_RPI: dv = (int32_t)floorf(cv.f + 0.5f); break;
+	case ALU_OP1_FLT_TO_INT_TRUNC: dv = (int32_t)truncf(cv.f); break;
 	case ALU_OP1_FLT_TO_UINT: dv = (uint32_t)cv.f; break;
-	case ALU_OP1_FRACT: dv = cv.f - floor(cv.f); break;
+	case ALU_OP1_FRACT: dv = cv.f - floorf(cv.f); break;
 	case ALU_OP1_INT_TO_FLT: dv = (float)cv.i; break;
 	case ALU_OP1_LOG_CLAMPED:
 	case ALU_OP1_LOG_IEEE:
 		if (cv.f != 0.0f)
-			dv = log2(cv.f);
+			dv = log2f(cv.f);
 		else
 			// don't fold to NAN, let the GPU handle it for now
 			// (prevents degenerate LIT tests from failing)
@@ -454,16 +442,21 @@ bool expr_handler::fold_alu_op1(alu_node& n) {
 	case ALU_OP1_PRED_SET_RESTORE: dv = cv; break;
 	case ALU_OP1_RECIPSQRT_CLAMPED:
 	case ALU_OP1_RECIPSQRT_FF:
-	case ALU_OP1_RECIPSQRT_IEEE: dv = 1.0f / sqrt(cv.f); break;
+	case ALU_OP1_RECIPSQRT_IEEE: dv = 1.0f / sqrtf(cv.f); break;
 	case ALU_OP1_RECIP_CLAMPED:
 	case ALU_OP1_RECIP_FF:
 	case ALU_OP1_RECIP_IEEE: dv = 1.0f / cv.f; break;
 //	case ALU_OP1_RECIP_INT:
-	case ALU_OP1_RECIP_UINT: dv.u = (1ull << 32) / cv.u; break;
-//	case ALU_OP1_RNDNE: dv = floor(cv.f + 0.5f); break;
+	case ALU_OP1_RECIP_UINT: {
+		if (!cv.u)
+			return false;
+		dv.u = (1ull << 32) / cv.u;
+		break;
+	}
+	//	case ALU_OP1_RNDNE: dv = floor(cv.f + 0.5f); break;
 	case ALU_OP1_SIN: dv = sin(cv.f * 2.0f * M_PI); break;
-	case ALU_OP1_SQRT_IEEE: dv = sqrt(cv.f); break;
-	case ALU_OP1_TRUNC: dv = trunc(cv.f); break;
+	case ALU_OP1_SQRT_IEEE: dv = sqrtf(cv.f); break;
+	case ALU_OP1_TRUNC: dv = truncf(cv.f); break;
 
 	default:
 		return false;
@@ -719,7 +712,7 @@ bool expr_handler::fold_assoc(alu_node *n) {
 			n->src[0] = n->src[2];
 			n->bc.src[0] = n->bc.src[2];
 			n->src[1] = sh.get_const_value(cr);
-			memset(&n->bc.src[1], 0, sizeof(bc_alu_src));
+			n->bc.src[1].clear();
 
 			n->src.resize(2);
 			n->bc.set_op(ALU_OP2_ADD);
@@ -729,7 +722,7 @@ bool expr_handler::fold_assoc(alu_node *n) {
 		n->bc.src[0] = a->bc.src[last_arg];
 		n->bc.src[0].neg ^= cur_neg;
 		n->src[1] = sh.get_const_value(cr);
-		memset(&n->bc.src[1], 0, sizeof(bc_alu_src));
+		n->bc.src[1].clear();
 	}
 
 	return false;
@@ -770,7 +763,7 @@ bool expr_handler::fold_alu_op2(alu_node& n) {
 			case ALU_OP2_ADD:  // (ADD x, x) => (MUL x, 2)
 				if (!sh.safe_math) {
 					n.src[1] = sh.get_const_value(2.0f);
-					memset(&n.bc.src[1], 0, sizeof(bc_alu_src));
+					n.bc.src[1].clear();
 					n.bc.set_op(ALU_OP2_MUL);
 					return fold_alu_op2(n);
 				}
@@ -1014,7 +1007,7 @@ bool expr_handler::fold_alu_op3(alu_node& n) {
 			value *mv0 = md->src[0]->gvalue();
 			value *mv1 = md->src[1]->gvalue();
 
-			int es0 = -1, es1;
+			int es0 = -1, es1 = -1;
 
 			if (v0 == mv0) {
 				es0 = 0;
@@ -1070,7 +1063,7 @@ bool expr_handler::fold_alu_op3(alu_node& n) {
 				}
 
 				n.src[1] = t;
-				memset(&n.bc.src[1], 0, sizeof(bc_alu_src));
+				n.bc.src[1].clear();
 
 				n.src.resize(2);
 
@@ -1101,7 +1094,7 @@ bool expr_handler::fold_alu_op3(alu_node& n) {
 				dv = cv0.f * cv1.f;
 				n.bc.set_op(ALU_OP2_ADD);
 				n.src[0] = sh.get_const_value(dv);
-				memset(&n.bc.src[0], 0, sizeof(bc_alu_src));
+				n.bc.src[0].clear();
 				n.src[1] = n.src[2];
 				n.bc.src[1] = n.bc.src[2];
 				n.src.resize(2);
