@@ -49,7 +49,7 @@ struct svga_winsys_screen;
 struct svga_winsys_buffer;
 struct pipe_screen;
 struct pipe_context;
-struct pipe_debug_callback;
+struct util_debug_callback;
 struct pipe_fence_handle;
 struct pipe_resource;
 struct svga_region;
@@ -81,8 +81,9 @@ struct winsys_handle;
 #define SVGA_FENCE_FLAG_EXEC      (1 << 0)
 #define SVGA_FENCE_FLAG_QUERY     (1 << 1)
 
-#define SVGA_SURFACE_USAGE_SHARED  (1 << 0)
-#define SVGA_SURFACE_USAGE_SCANOUT (1 << 1)
+#define SVGA_SURFACE_USAGE_SHARED   (1 << 0)
+#define SVGA_SURFACE_USAGE_SCANOUT  (1 << 1)
+#define SVGA_SURFACE_USAGE_COHERENT (1 << 2)
 
 #define SVGA_QUERY_FLAG_SET        (1 << 0)
 #define SVGA_QUERY_FLAG_REF        (1 << 1)
@@ -98,6 +99,9 @@ struct svga_winsys_stats_timeframe {
    uint64 startTime;
    uint64 adjustedStartTime;
    struct svga_winsys_stats_timeframe *enclosing;
+
+   struct svga_winsys_screen *sws;
+   int32 slot;
 };
 
 enum svga_stats_count {
@@ -105,6 +109,7 @@ enum svga_stats_count {
    SVGA_STATS_COUNT_BLITBLITTERCOPY,
    SVGA_STATS_COUNT_DEPTHSTENCILSTATE,
    SVGA_STATS_COUNT_RASTERIZERSTATE,
+   SVGA_STATS_COUNT_RAWBUFFERSRVIEW,
    SVGA_STATS_COUNT_SAMPLER,
    SVGA_STATS_COUNT_SAMPLERVIEW,
    SVGA_STATS_COUNT_SURFACEWRITEFLUSH,
@@ -127,10 +132,13 @@ enum svga_stats_time {
    SVGA_STATS_TIME_CREATEBACKEDSURFACEVIEW,
    SVGA_STATS_TIME_CREATEBUFFER,
    SVGA_STATS_TIME_CREATECONTEXT,
+   SVGA_STATS_TIME_CREATECS,
    SVGA_STATS_TIME_CREATEFS,
    SVGA_STATS_TIME_CREATEGS,
    SVGA_STATS_TIME_CREATESURFACE,
    SVGA_STATS_TIME_CREATESURFACEVIEW,
+   SVGA_STATS_TIME_CREATETCS,
+   SVGA_STATS_TIME_CREATETES,
    SVGA_STATS_TIME_CREATETEXTURE,
    SVGA_STATS_TIME_CREATEVS,
    SVGA_STATS_TIME_DEFINESHADER,
@@ -138,8 +146,12 @@ enum svga_stats_time {
    SVGA_STATS_TIME_DRAWVBO,
    SVGA_STATS_TIME_DRAWARRAYS,
    SVGA_STATS_TIME_DRAWELEMENTS,
+   SVGA_STATS_TIME_EMITCS,
    SVGA_STATS_TIME_EMITFS,
    SVGA_STATS_TIME_EMITGS,
+   SVGA_STATS_TIME_EMITRAWBUFFER,
+   SVGA_STATS_TIME_EMITTCS,
+   SVGA_STATS_TIME_EMITTES,
    SVGA_STATS_TIME_EMITVS,
    SVGA_STATS_TIME_EMULATESURFACEVIEW,
    SVGA_STATS_TIME_FENCEFINISH,
@@ -148,6 +160,7 @@ enum svga_stats_time {
    SVGA_STATS_TIME_HWTNLDRAWELEMENTS,
    SVGA_STATS_TIME_HWTNLFLUSH,
    SVGA_STATS_TIME_HWTNLPRIM,
+   SVGA_STATS_TIME_LAUNCHGRID,
    SVGA_STATS_TIME_PROPAGATESURFACE,
    SVGA_STATS_TIME_SETSAMPLERVIEWS,
    SVGA_STATS_TIME_SURFACEFLUSH,
@@ -158,7 +171,9 @@ enum svga_stats_time {
    SVGA_STATS_TIME_TEXTRANSFERUNMAP,
    SVGA_STATS_TIME_TGSIVGPU10TRANSLATE,
    SVGA_STATS_TIME_TGSIVGPU9TRANSLATE,
+   SVGA_STATS_TIME_UPDATECSUAV,
    SVGA_STATS_TIME_UPDATESTATE,
+   SVGA_STATS_TIME_UPDATEUAV,
    SVGA_STATS_TIME_VALIDATESURFACEVIEW,
    SVGA_STATS_TIME_VBUFDRAWARRAYS,
    SVGA_STATS_TIME_VBUFDRAWELEMENTS,
@@ -176,6 +191,7 @@ enum svga_stats_time {
    SVGA_STATS_PREFIX "BlitBlitterCopy",       \
    SVGA_STATS_PREFIX "DepthStencilState",     \
    SVGA_STATS_PREFIX "RasterizerState",       \
+   SVGA_STATS_PREFIX "RawBufferSRView",       \
    SVGA_STATS_PREFIX "Sampler",               \
    SVGA_STATS_PREFIX "SamplerView",           \
    SVGA_STATS_PREFIX "SurfaceWriteFlush",     \
@@ -196,10 +212,13 @@ enum svga_stats_time {
    SVGA_STATS_PREFIX "CreateBackedSurfaceView",     \
    SVGA_STATS_PREFIX "CreateBuffer",                \
    SVGA_STATS_PREFIX "CreateContext",               \
+   SVGA_STATS_PREFIX "CreateCS",                    \
    SVGA_STATS_PREFIX "CreateFS",                    \
    SVGA_STATS_PREFIX "CreateGS",                    \
    SVGA_STATS_PREFIX "CreateSurface",               \
    SVGA_STATS_PREFIX "CreateSurfaceView",           \
+   SVGA_STATS_PREFIX "CreateTCS",                   \
+   SVGA_STATS_PREFIX "CreateTES",                   \
    SVGA_STATS_PREFIX "CreateTexture",               \
    SVGA_STATS_PREFIX "CreateVS",                    \
    SVGA_STATS_PREFIX "DefineShader",                \
@@ -207,8 +226,12 @@ enum svga_stats_time {
    SVGA_STATS_PREFIX "DrawVBO",                     \
    SVGA_STATS_PREFIX "DrawArrays",                  \
    SVGA_STATS_PREFIX "DrawElements",                \
+   SVGA_STATS_PREFIX "EmitCS",                      \
    SVGA_STATS_PREFIX "EmitFS",                      \
    SVGA_STATS_PREFIX "EmitGS",                      \
+   SVGA_STATS_PREFIX "EmitRawBuffer",               \
+   SVGA_STATS_PREFIX "EmitTCS",                     \
+   SVGA_STATS_PREFIX "EmitTES",                     \
    SVGA_STATS_PREFIX "EmitVS",                      \
    SVGA_STATS_PREFIX "EmulateSurfaceView",          \
    SVGA_STATS_PREFIX "FenceFinish",                 \
@@ -217,6 +240,7 @@ enum svga_stats_time {
    SVGA_STATS_PREFIX "HWtnlDrawElements",           \
    SVGA_STATS_PREFIX "HWtnlFlush",                  \
    SVGA_STATS_PREFIX "HWtnlPrim",                   \
+   SVGA_STATS_PREFIX "LaunchGrid",                  \
    SVGA_STATS_PREFIX "PropagateSurface",            \
    SVGA_STATS_PREFIX "SetSamplerViews",             \
    SVGA_STATS_PREFIX "SurfaceFlush",                \
@@ -227,7 +251,9 @@ enum svga_stats_time {
    SVGA_STATS_PREFIX "TextureTransferUnmap",        \
    SVGA_STATS_PREFIX "TGSIVGPU10Translate",         \
    SVGA_STATS_PREFIX "TGSIVGPU9Translate",          \
+   SVGA_STATS_PREFIX "UpdateCSUAV",                 \
    SVGA_STATS_PREFIX "UpdateState",                 \
+   SVGA_STATS_PREFIX "UpdateUAV",                   \
    SVGA_STATS_PREFIX "ValidateSurfaceView",         \
    SVGA_STATS_PREFIX "VbufDrawArrays",              \
    SVGA_STATS_PREFIX "VbufDrawElements",            \
@@ -385,21 +411,27 @@ struct svga_winsys_context
     **/
 
    boolean have_gb_objects;
+   boolean force_coherent;
 
    /**
     * Map a guest-backed surface.
-    * \param flags  bitmaks of PIPE_TRANSFER_x flags
+    * \param swc The winsys context
+    * \param surface The surface to map
+    * \param flags  bitmask of PIPE_MAP_x flags
+    * \param retry Whether to flush and retry the map
+    * \param rebind Whether to issue an immediate rebind and flush.
     *
     * The surface_map() member is allowed to fail due to a
     * shortage of command buffer space, if the
-    * PIPE_TRANSFER_DISCARD_WHOLE_RESOURCE bit is set in flags.
+    * PIPE_MAP_DISCARD_WHOLE_RESOURCE bit is set in flags.
     * In that case, the caller must flush the current command
     * buffer and reissue the map.
     */
    void *
    (*surface_map)(struct svga_winsys_context *swc,
                   struct svga_winsys_surface *surface,
-                  unsigned flags, boolean *retry);
+                  unsigned flags, boolean *retry,
+                  boolean *rebind);
 
    /**
     * Unmap a guest-backed surface.
@@ -411,13 +443,6 @@ struct svga_winsys_context
                     struct svga_winsys_surface *surface,
                     boolean *rebind);
 
-   /**
-    * Invalidate the content of this surface
-    */
-   enum pipe_error
-   (*surface_invalidate)(struct svga_winsys_context *swc,
-                         struct svga_winsys_surface *surface);
-
    /**
     * Create and define a DX GB shader that resides in the device COTable.
     * Caller of this function will issue the DXDefineShader command.
@@ -427,7 +452,9 @@ struct svga_winsys_context
                     uint32 shaderId,
                     SVGA3dShaderType shaderType,
                     const uint32 *bytecode,
-                    uint32 bytecodeLen);
+                    uint32 bytecodeLen,
+                    const SVGA3dDXShaderSignatureHeader *sgnInfo,
+                    uint32 sgnLen);
 
    /**
     * Destroy a DX GB shader.
@@ -449,15 +476,21 @@ struct svga_winsys_context
                       struct svga_winsys_gb_shader *shader,
                       unsigned flags);
 
-   /** To report perf/conformance/etc issues to the state tracker */
-   struct pipe_debug_callback *debug_callback;
+   /** To report perf/conformance/etc issues to the gallium frontend */
+   struct util_debug_callback *debug_callback;
 
    /** The more recent command issued to command buffer */
    SVGAFifo3dCmdId last_command;
 
    /** For HUD queries */
    uint64_t num_commands;
+   uint64_t num_command_buffers;
    uint64_t num_draw_commands;
+   uint64_t num_shader_reloc;
+   uint64_t num_surf_reloc;
+
+   /* Whether we are in retry processing */
+   unsigned int in_retry;
 };
 
 
@@ -586,6 +619,11 @@ struct svga_winsys_screen
                          uint32 numMipLevels,
                          uint32 numSamples);
 
+   void
+   (*surface_init)(struct svga_winsys_screen *sws,
+                   struct svga_winsys_surface *surface,
+                   unsigned surf_size, SVGA3dSurfaceAllFlags flags);
+
    /**
     * Buffer management. Buffer attributes are mostly fixed over its lifetime.
     *
@@ -602,7 +640,7 @@ struct svga_winsys_screen
 
    /**
     * Map the entire data store of a buffer object into the client's address.
-    * usage is a bitmask of PIPE_TRANSFER_*
+    * usage is a bitmask of PIPE_MAP_*
     */
    void *
    (*buffer_map)( struct svga_winsys_screen *sws,
@@ -683,6 +721,8 @@ struct svga_winsys_screen
    /** Can we do DMA with guest-backed objects enabled? */
    bool have_gb_dma;
 
+   /** Do we support coherent surface memory? */
+   bool have_coherent;
    /**
     * Create and define a GB shader.
     */
@@ -739,19 +779,19 @@ struct svga_winsys_screen
     * Increment a statistic counter
     */
    void
-   (*stats_inc)(enum svga_stats_count);
+   (*stats_inc)(struct svga_winsys_screen *, enum svga_stats_count);
 
    /**
     * Push a time frame onto the stack
     */
    void
-   (*stats_time_push)(enum svga_stats_time, struct svga_winsys_stats_timeframe *);
+   (*stats_time_push)(struct svga_winsys_screen *, enum svga_stats_time, struct svga_winsys_stats_timeframe *);
 
    /**
     * Pop a time frame.
     */
    void
-   (*stats_time_pop)();
+   (*stats_time_pop)(struct svga_winsys_screen *);
 
    /**
     * Send a host log message
@@ -765,7 +805,10 @@ struct svga_winsys_screen
    /** Have SM4_1 hardware? */
    boolean have_sm4_1;
 
-   /** To rebind resources at the beginnning of a new command buffer */
+   /** Have SM5 hardware? */
+   boolean have_sm5;
+
+   /** To rebind resources at the beginning of a new command buffer */
    boolean need_to_rebind_resources;
 
    boolean have_generate_mipmap_cmd;
@@ -773,6 +816,17 @@ struct svga_winsys_screen
    boolean have_transfer_from_buffer_cmd;
    boolean have_fence_fd;
    boolean have_intra_surface_copy;
+   boolean have_constant_buffer_offset_cmd;
+   boolean have_index_vertex_buffer_offset_cmd;
+
+   /* Have rasterizer state v2 command support */
+   boolean have_rasterizer_state_v2_cmd;
+
+   /** Have GL43 capable device */
+   boolean have_gl43;
+
+   /** SVGA device_id version we're running on */
+   uint16_t device_id;
 };
 
 
